﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GameController : MonoBehaviour {

	public Transform Player;

	public Texture MenuTitleTexture;
	public Texture LevelTexture;
	public Texture YouLoseTexture;
	public Texture YouWinTexture;
	public Texture TutorialTexture;


	public List<AudioClip> SplashUpSounds;
	public List<AudioClip> SplashDownSounds;
	public AudioClip ThrustSoundUnderwater;
	public AudioClip ThrustSoundAir;
	public AudioClip AlienCaptureSound;
	public AudioClip PlayerCrashSound;
	public AudioClip BoundaryBumpSound;
	public AudioClip CrossFinishLineSound;
	public AudioClip ScreenChangeSound;

	private AudioSource asrc;


	public Font GUIFont;
	public int GUIFontSize;
	private GUIStyle levelNumStyle;
	private bool guiInitialized=false;

	public List<Transform> AlienPrefabs;
	public List<Transform> IcebergPrefabs;
	public List<Transform> WaterLinePrefabs;
	public Transform FloorPrefab;
	public Transform UnderwaterBasePrefab;
	public Transform UnderwaterOverlayPrefab;
	public Transform FinishLinePrefab;

	public List<LevelParams> LevelParamsList;

	public LevelData CurrentLevel { get; set; }
	public System.Random rand { get; set; }
	public int TotalShipsWrecked { get; set; }
	public int TotalAliensCaptured { get; set; }
	public int TotalAliensSeen { get; set; }

	private enum GameState
	{
		MenuTitleScreen,
		LevelScreen,
		Running,
		YouLoseScreen,
		YouWinScreen,
		TutorialScreen
	}
	private GameState state;
	private float lastSwitchTime;

	private LevelGenerator levelGenerator;
	private LevelInstantiator levelInstantiator;

	void Start () 
	{
		asrc = gameObject.AddComponent<AudioSource>();

		rand = new System.Random();
		levelNumStyle = new GUIStyle();
		levelNumStyle.normal.textColor = Color.white;
		levelNumStyle.fontSize = 60;
		levelNumStyle.font = GUIFont;

		CurrentLevel = new LevelData(0);
		levelGenerator = new LevelGenerator(this);
		levelInstantiator = new LevelInstantiator(this);

		EnterState (GameState.MenuTitleScreen);
	}
	
	void Update () 
	{
		switch(state)
		{
		case GameState.MenuTitleScreen:
			if(Input.GetKeyDown(KeyCode.Space))
			{
				CurrentLevel = levelGenerator.GenerateLevel(1);
				PlaySound(ScreenChangeSound, 1.0f);
				EnterState(GameState.LevelScreen);
			}
			break;

		case GameState.LevelScreen:
			if(Time.time - lastSwitchTime > 0.1 && (Input.GetKeyDown(KeyCode.Space)))
			{
				CurrentLevel.Over = false;
				CurrentLevel.AliensCaptured = 0;
				levelInstantiator.InstantiateLevel(CurrentLevel);
				PlaySound(ScreenChangeSound, 1.0f);
				EnterState(GameState.Running);
			}
			break;

		case GameState.Running:
						
			if(CurrentLevel.Over)
			{
				foreach(Transform t in CurrentLevel.TransformsForCleanup)
				{
					GameObject.Destroy (t.gameObject);
				}
				CurrentLevel.TransformsForCleanup.Clear ();

				if(CurrentLevel.Victory)
				{
					TotalAliensCaptured += CurrentLevel.AliensCaptured;
					TotalAliensSeen += CurrentLevel.AlienCount;

					EnterState (GameState.YouWinScreen);
				}
				else
				{
					EnterState (GameState.YouLoseScreen);
				}
			}
			break;

		case GameState.YouWinScreen:
			if(Time.time - lastSwitchTime > 0.65 && (Input.GetKeyDown(KeyCode.Space)))
			{
				CurrentLevel = levelGenerator.GenerateLevel(CurrentLevel.Level + 1);
				PlaySound(ScreenChangeSound, 1.0f);
				EnterState(GameState.LevelScreen);
			}

			break;

		case GameState.YouLoseScreen:
			if(Time.time - lastSwitchTime > 0.15 && (Input.GetKeyDown(KeyCode.Space)))
			{
				PlaySound(ScreenChangeSound, 1.0f);
				EnterState(GameState.LevelScreen);
			}
			else if(Time.time - lastSwitchTime > 0.25 && (Input.GetKeyDown(KeyCode.Q)))
			{
				TotalAliensSeen = 0;
				TotalShipsWrecked = 0;
				TotalAliensCaptured = 0;

				PlaySound(ScreenChangeSound, 1.0f);
				EnterState(GameState.MenuTitleScreen);
			}
			break;

		case GameState.TutorialScreen:
			if(Input.GetMouseButtonDown(0))
			{
				EnterState(GameState.MenuTitleScreen);
			}
			break;

		}
	}

	void OnGUI()
	{
		if(!guiInitialized)
		{
			GUI.skin.label.font = GUIFont;
			GUI.skin.label.fontSize = GUIFontSize;
			guiInitialized=true;
		}
				
		switch(state)
		{
		case GameState.MenuTitleScreen:
			GUI.DrawTexture(new Rect(0,0, 1024,600), MenuTitleTexture);
			break;
		case GameState.LevelScreen:
			GUI.DrawTexture(new Rect(0,0, 1024,600),  LevelTexture);
			GUI.Label(new Rect(480,140,40,40), "" + CurrentLevel.Level, levelNumStyle);
			GUI.Label(new Rect(250,325,1024-400,200), GetTipText());
			break;
		case GameState.Running:
			// TODO ADD HUD CODE HERE
			break;
		case GameState.YouWinScreen:
			GUI.DrawTexture(new Rect(0,0, 1024,600),  YouWinTexture);
			// TODO Display results here
			string s;
			if(CurrentLevel.AlienCount == 0)
			{
				if(CurrentLevel.Level == 1)
					s = "Good job.  Avoid hitting the ceiling and floor, or you will lose control for a moment.\n\n\nWARNING: Dangerous icebergs detected ahead.";
				else
				{
					s = "\nShips Wrecked on Mission: " + CurrentLevel.ShipsWrecked +"\n\n";

					if(CurrentLevel.ShipsWrecked ==0)
						s += "Excellent!  Have you done this before?\n\n";
					else if(CurrentLevel.ShipsWrecked < 3)
						s += "Good work, a few wrecks is normal.\n\n";
					else
						s += "Hang in there, it takes a few crashes and then you'll get the hang of it!  We've brought plenty of spare ships.\n\n";

					if(CurrentLevel.Level == 2)
						s += "\nReading possible life signs ahead.\nWe're sending you to investigate.";
				}
			}
			else 
			{
				s = "Specimens Acquired on Mission " + CurrentLevel.Level + ": " + CurrentLevel.AliensCaptured + " / " + CurrentLevel.AlienCount + "\n\n";
				float pct = (float)CurrentLevel.AliensCaptured / (float)CurrentLevel.AlienCount;
				if(pct == 1.0f)
					s += "100%?!  Is that even possible?!\n";
				else if(pct >=0.8f)
					s += "INCREDIBLE!\n";
				else if(pct >=0.7f)
					s += "Astounding - the Science Team can't figure out how you did so well\n";
				else if(pct >=0.6f)
					s += "Wow!  We're showing that tape to the trainees\n";
				else if(pct >=0.6f)
					s += "Ace flying, we're taking notes up here\n";
				else if(pct >=0.5f)
					s += "Top-notch field work!\n";
				else if(pct >=0.45f)
					s += "Very impressive\n";
				else if(pct >= 0.4f)
					s += "Nice flying, pilot\n";
				else if(pct >= 0.35f)
					s += "Lots to work with here\n";
				else if(pct >= 0.3f)
					s += "Pretty good job, considering the risks\n";
				else if(pct >=0.25f)
					s += "Fascinating, please acquire more\n";
				else if(pct >=0.2f)
					s += "Good, but we really wanted that one that got away\n";
				else if(pct >=0.1f)
					s += "More questions than answers\n";
				else if(pct > 0f)
					s += "Needs improvement.  Our funding is at stake.\n";
				else
					s += "Focus on your mission!  Capture specimens.\n";

				s += "\n\n\nTotal Specimens Acquired: " + TotalAliensCaptured + " / " + TotalAliensSeen + "\n";

				s += "\nShips Wrecked on Mission: " + CurrentLevel.ShipsWrecked + "\n";
				s += "\nTotal Ships Wrecked: " + TotalShipsWrecked + "\n";
			}

			GUI.Label(new Rect(100,250,1024-200,300), s);

			break;
		case GameState.YouLoseScreen:
			GUI.DrawTexture(new Rect(0,0, 1024,600),  YouLoseTexture);
			break;
		case GameState.TutorialScreen:
			GUI.DrawTexture(new Rect(5,5, 600,600),  TutorialTexture);
			break;
			
		}
	}

	private string GetTipText()
	{
		switch(CurrentLevel.Level)
		{
		case 1:
			return "Before things get serious, take a quick practice flight in air and water\n\n\nPress W/S or Up/Down to fire jets\n\nYou can't fail this one!";
			break;

		case 2:
			return "Avoid the icebergs!\n\n\nIn the air, you fall\n\nIn the water, you float";
			break;

		case 3:
			return "ALIEN LIFE FORMS DETECTED!\n\nAcquire specimens by touching them.  You won't hurt them.";
			break;

		case 4:
			return "Hitting the ceiling or floor will knock your steering jets out for a moment.\n\n\nCapture those aliens!";
			break;
		
		case 5:
			return "Your jets cannot overcome the forces of nature\n\nLook ahead and shape your path early.  Planning is essential.";
			break;

		case 6:
			return "Flying in big high arcs gets you far from icebergs\n\nBut try quick low arcs, they keep more options open";
			break;

		case 7:
			return "You can get closer to icebergs without crashing than you might think.\n\nAliens are snatched up from a bit farther away";
			break;

		case 8:
			return "When you retry a level, it's the same.\n\nBut if you go to the main menu to start over, all levels are re-randomized.";
			break;

		case 9:
			return "The Science Team thanks you for your service.\n\nYou can email them at ricefrog@gmail.com";
			break;

		case 10:
			return "     Become the dolphin.";
			break;


		default:
			return "There's no additional unseen content, but plz enjoy an infinite supply of shorter tougher random levels.\n\nGood luck out there!";
			break;
		}
	}

	private void EnterState(GameState newState)
	{
		lastSwitchTime = Time.time;
		GameState oldState = state;
		state = newState;

		Debug.Log ("GameController Entering State " + state + " from " + oldState);

		switch(state)
		{
		case GameState.MenuTitleScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.LevelScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.Running:
			Player.gameObject.SetActive(true);
			Player.position = CurrentLevel.StartPosition;
			break;
		case GameState.YouWinScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.YouLoseScreen:
			Player.gameObject.SetActive(false);
			break;
		case GameState.TutorialScreen:
			Player.gameObject.SetActive(false);
			break;
			
		}
	}

	public void PlayUpSplash()
	{
		int s = rand.Next(0, SplashUpSounds.Count);
		PlaySound(SplashUpSounds[s], 1.0f);
	}

	public void PlayDownSplash()
	{
		int s = rand.Next(0, SplashDownSounds.Count);
		PlaySound(SplashDownSounds[s], 1.0f);
	}

	public void PlaySound(AudioClip clip, float vol)
	{
		asrc.PlayOneShot(clip, vol);
	}

}
